package abstractmath

import abstractmath.Real.Companion.R0

typealias ComplexOver<El> = EllipticCayleyDicksonConstruction<El>

typealias Complex = ComplexOver<Real>
typealias DualNumber = ParabolicCayleyDicksonConstruction<Real>
typealias SplitComplex = HyperbolicCayleyDicksonConstruction<Real>

typealias Quaternion = ComplexOver<Complex>
typealias Octonion = ComplexOver<Quaternion>
typealias Sedenion = ComplexOver<Octonion>

fun complexOf(real: Number, imaginary: Number) = Complex(real.R, imaginary.R)

val Number.i: Complex get() = Complex(R0, this.R)

fun quaternionOf(real: Number, i: Number, j: Number, k: Number) =
        Quaternion(Complex(real.R, i.R), Complex(j.R, k.R))

fun quaternionOf(real: Real, i: Real, j: Real, k: Real) =
        Quaternion(Complex(real, i), Complex(j, k))

val Number.j: Quaternion get() = quaternionOf(0, 0, this, 0)
val Number.k: Quaternion get() = quaternionOf(0, 0, 0, this)

val Number.eps: DualNumber get() = DualNumber(R0, this.R)

typealias EVector = EuclideanVector<Real>

operator fun EVector.times(x: Number) = times(x.R)
operator fun EVector.div(x: Number) = div(x.R)
operator fun Number.times(x: EVector) = x.times(this.R)