@file:Suppress("unused")

package abstractmath

import java.math.BigInteger as BInt

class Real(val x: Double) : Field<Real>, Comparable<Real> {
    companion object {
        val arr = List(33) { Real(it - 16.0) }
        val R0 = 0.R
        val R1 = 1.R
        val Rm1 = (-1).R
    }

    override operator fun plus(another: Real) = Real(x + another.x)

    override operator fun unaryMinus() = Real(-x)

    override operator fun minus(another: Real) = Real(x - another.x)

    override operator fun times(another: Real) = Real(x * another.x)

    override fun inverse() = Real(1.0 / x)

    override operator fun div(another: Real) = Real(x / another.x)

    override fun involution() = this

    override operator fun compareTo(other: Real) = x.compareTo(other.x)

    override fun equals(other: Any?) = other is Real && compareTo(other) == 0

    override fun toString() = "$x"

    override fun zero() = R0

    override fun one() = R1

    fun toDouble() = x
}

val Number.R: Real
    get() {
        if (this is Int && -16 <= this && this <= 16) return Real.Companion.arr[this + 16]
        if (this is Short && -16 <= this && this <= 16) return Real.Companion.arr[this + 16]
        if (this is Long && -16 <= this && this <= 16) return Real.Companion.arr[this.toInt() + 16]
        if (this is Byte && -16 <= this && this <= 16) return Real.Companion.arr[this + 16]
        return Real(this.toDouble())
    }

val BInt.R: Real get() = Real(toDouble())